# Copyright (C) 2003, 2004 Gregoire Lejeune <gregoire.lejeune@free.fr>
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

require 'tempfile'

require 'graphviz/node'
require 'graphviz/edge'
require 'graphviz/attrs'
require 'graphviz/constants'

class GraphViz 
  include Constants

  public

  ## Var: Output format (dot, png, jpeg, ...)
  @format
  ## Var: Output file name
  @filename

  ## Var: Graph name
  @name

  ## Var: defined attributs
  @graph
  @node
  @edge
  attr_accessor :node, :edge

  def add_node( xNodeName, *hOpt )
    @hoNodes[xNodeName] = GraphViz::Node::new( xNodeName )
   
    if hOpt.nil? == false and hOpt[0].nil? == false
      hOpt[0].each do |xKey, xValue|
	    @hoNodes[xNodeName][xKey] = xValue
	  end
	end
	
    return( @hoNodes[xNodeName] )
  end

  def add_edge( oNodeOne, oNodeTwo, *hOpt )
    oEdge = GraphViz::Edge::new( oNodeOne, oNodeTwo )
    
	if hOpt.nil? == false and hOpt[0].nil? == false
      hOpt[0].each do |xKey, xValue|
	    oEdge[xKey] = xValue
	  end
	end
	
    @loEdges.push( oEdge )
    return( oEdge )
  end

  def add_graph( xGraphName, *hOpt )
    @hoGraphs[xGraphName] = GraphViz::new( xGraphName, "parent" => self )
   
    if hOpt.nil? == false and hOpt[0].nil? == false
      hOpt[0].each do |xKey, xValue|
	    @hoGraphs[xGraphName][xKey] = xValue
	  end
	end
	
    return( @hoGraphs[xGraphName] )
  end

  def []=( xAttrName, xValue )
    @graph[xAttrName] = xValue
  end

  def []( xAttrName )
    return( @graph[xAttrName].clone )
  end
  
  def output( *hOpt )
    xDOTScript = ""

    ## Add default attributs' graph values
    graph = ""
    separator = ""
    @graph.data.each do |k,v|
      graph << separator + k + " = \"" + v + "\""
      separator = "; "
    end
    if graph.length > 0 
      xDOTScript << "  " + graph + ";\n"
    end

    ## Add default attributs' node values
    node = ""
    separator = ""
    @node.data.each do |k,v|
      node << separator + k + " = \"" + v + "\""
      separator = ", "
    end
    if node.length > 0 
      xDOTScript << "  node [" + node + "];\n"
    end

    ## Add default attributs' edge values
    edge = ""
    separator = ""
    @edge.data.each do |k,v|
      edge << separator + k + " = \"" + v + "\""
      separator = ", "
    end
    if edge.length > 0 
      xDOTScript << "  edge [" + edge + "];\n"
    end

    # graphs
	@hoGraphs.each do |xName, oGraph|
	  xDOTScript << oGraph.output() + "\n"
	end
    
    # nodes
    @hoNodes.each do |xName, oNode|
      xDOTScript << "  " + oNode.output() + "\n"
    end
    
    # edges
    @loEdges.each do |oEdge|
      xDOTScript << "  " + oEdge.output() + "\n"
    end

    xDOTScript << "}"
      
    if @oParentGraph.nil? == false
      xDOTScript = "subgraph #{@name} {\n" << xDOTScript

      return( xDOTScript )
    else
      if hOpt.nil? == false and hOpt[0].nil? == false
        hOpt[0].each do |xKey, xValue|
          case xKey
            when "output"
              if FORMATS.index( xValue ).nil? == true
                raise ArgumentError, "output format '#{xValue}' invalid"
              end
              @format = xValue
            when "file"
              @filename = xValue
            else
              raise ArgumentError, "option #{xKey} unknown"
          end
        end
      end
  
      xDOTScript = "digraph #{@name} {\n" << xDOTScript

      ## Act: Save script and send it to dot
      t = Tempfile::open( File.basename($0) + "." )
      t.print( xDOTScript )
      t.close
      
      cmd = find_executable( "dot" )
	  if cmd == nil
        raise StandardError, "GraphViz not installed"
	  end
      
      xFile = ""
      xFile = "-o #{@filename}" if @filename.nil? == false
      xCmd = "#{cmd} -T#{@format} #{xFile} #{t.path}"
      
      f = IO.popen( xCmd )
      print f.readlines
      f.close
    end
  end
  
## ----------------------------------------------------------------------------

  private 

  ## Var: Nodes, Edges and Graphs tables
  @hoNodes
  @loEdges
  @hoGraphs

  ## Var: Parent graph
  @oParentGraph
  
  def initialize( xGraphName, *hOpt )
    @format   = "dot"
    @filename = nil
    @name = xGraphName

    @oParentGraph = nil
    
    @hoNodes  = Hash::new()
    @loEdges  = Array::new()
    @hoGraphs = Hash::new()
    
    @node  = GraphViz::Attrs::new( "node",  NODESATTRS  )
    @edge  = GraphViz::Attrs::new( "edge",  EDGESATTRS  )
    @graph = GraphViz::Attrs::new( "graph", GRAPHSATTRS )

    if hOpt.nil? == false and hOpt[0].nil? == false
      hOpt[0].each do |xKey, xValue|
        case xKey
          when "output"
            if FORMATS.index( xValue ).nil? == true
              raise ArgumentError, "output format '#{xValue}' invalid"
            end
            @format = xValue
          when "file"
            @filename = xValue
          when "parent"
            @oParentGraph = xValue
          else
		    self[xKey] = xValue
        end
      end
    end
  end
  
  def find_executable(bin, path = nil)
    path = (path || ENV['PATH']).split(File::PATH_SEPARATOR)
    file = nil
    path.each do |dir|
      return file if File.executable?(file = File.join(dir, bin))
    end
    nil
  end
end

